@@ -8,6 +8,7 @@ from unittest import mock
 
 from posixpath import join
 from urllib.parse import urljoin
+from html import unescape
 import pyotp
 import pytest
 
@@ -177,7 +178,7 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
         update_data = {
             "id": 1,
             "name": "Agent test",
-            "is_online": True,
+            "sid": "super_sid",
         }
         res = test_client.put(self.url(agent.id), data=update_data)
         assert res.status_code == 200, (res.json, update_data)
@@ -234,6 +235,7 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
 
     def test_run_agent_invalid_executor_argument(self, session, test_client):
         agent = AgentFactory.create()
+        agent.sid = "this_is_a_sid"
         executor = ExecutorFactory.create(agent=agent)
         workspace = WorkspaceFactory.create()
 
@@ -293,6 +295,7 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
 
     def test_invalid_executor(self, test_client, session, csrf_token):
         agent = AgentFactory.create()
+        agent.sid = "this_is_a_sid"
         workspace = WorkspaceFactory.create()
         session.add(agent)
         session.commit()
@@ -314,6 +317,7 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
 
     def test_happy_path_valid_json(self, test_client, session, csrf_token):
         agent = AgentFactory.create()
+        agent.sid = "this_is_a_sid"
         executor = ExecutorFactory.create(agent=agent)
         executor2 = ExecutorFactory.create(agent=agent)
         workspace = WorkspaceFactory.create()
@@ -350,6 +354,7 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
 
     def test_invalid_parameter_type(self, test_client, session, csrf_token):
         agent = AgentFactory.create()
+        agent.sid = "this_is_a_sid"
         executor = ExecutorFactory.create(agent=agent)
         workspace = WorkspaceFactory.create()
 
@@ -413,3 +418,116 @@ class TestAgentAPIGeneric(ReadWriteAPITests):
         assert "BURP_API_PULL_INTERVAL" in res.json["burp"]["optional_environment_variables"]
         assert "TENABLE_PULL_INTERVAL" in res.json["tenableio"]["optional_environment_variables"]
         assert res.status_code == 200
+
+    @pytest.fixture
+    def executors(self, workspace):
+        """
+        Creates multiple executors with different parameters_metadata structures.
+        """
+        return [
+            factories.ExecutorFactory.create(
+                parameters_metadata={
+                    "NUCLEI_TARGET": {"base": "list", "type": "list", "mandatory": True},
+                    "NUCLEI_EXCLUDE": {"base": "list", "type": "list", "mandatory": False},
+                },
+            ),
+            factories.ExecutorFactory.create(
+                parameters_metadata={
+                    "TARGET_URL": {"base": "string", "type": "string", "mandatory": True},
+                    "NAMED_CONFIGURATION": {"base": "string", "type": "string", "mandatory": False},
+                },
+            ),
+            factories.ExecutorFactory.create(
+                parameters_metadata={
+                    "TOKEN": {"base": "string", "type": "string", "mandatory": True},
+                    "GET_HOTSPOT": {"base": "boolean", "type": "boolean", "mandatory": False},
+                    "COMPONENT_KEY": {"base": "string", "type": "string", "mandatory": False},
+                },
+            ),
+            factories.ExecutorFactory.create(
+                parameters_metadata={
+                    "DAYS_OLD": {"base": "string", "type": "string", "mandatory": True}
+                },
+            ),
+            factories.ExecutorFactory.create(
+                parameters_metadata={
+                    "SHODAN_QUERY": {"base": "string", "type": "string", "mandatory": True}
+                },
+            ),
+        ]
+
+    @pytest.mark.parametrize("executor_index", [0, 1, 2, 3, 4])
+    def test_save_parameters_success(self, test_client, session, executors, executor_index):
+        """
+        Ensures valid parameters are saved successfully for all executor types.
+        """
+        executor = executors[executor_index]
+        session.add(executor)
+        session.commit()
+
+        valid_data = {
+            "executor_id": executor.id,
+            "parameters_data": {
+                "executor_data": {
+                    "args": {
+                        key: "test_value" if meta["base"] == "string" else True if meta["base"] == "boolean" else [
+                            "test_item"]
+                        for key, meta in executor.parameters_metadata.items()  # Creates args dynamically based in parameters metadata
+                        if meta["mandatory"]
+                    }
+                }
+            },
+        }
+
+        response = test_client.post(
+            self.url() + '/save_parameters', json=valid_data, content_type="application/json"
+        )
+        assert response.status_code == 200
+        assert response.json["message"] == "Parameters saved successfully"
+
+    def test_save_parameters_data_missing_executor_id(self, test_client, session):
+        """Test missing executor_id returns 400."""
+        data = {"parameters_data": {"executor_data": {"args": {}}}}
+        response = test_client.post(self.url() + '/save_parameters', json=data, content_type="application/json")
+        assert response.status_code == 400
+        # Unescape HTML entities to check against raw text
+        response_text = unescape(response.data.decode('utf-8'))
+        assert "Missing 'executor_id'" in response_text
+
+    def test_save_parameters_data_missing_parameters_data(self, test_client, session):
+        """Test missing parameters_data returns 400."""
+        executor = factories.ExecutorFactory(parameters_metadata={})
+        session.add(executor)
+        session.commit()
+
+        data = {"executor_id": executor.id}
+        response = test_client.post(self.url() + '/save_parameters', json=data, content_type="application/json")
+        assert response.status_code == 400
+        response_text = unescape(response.data.decode('utf-8'))
+        assert "Missing 'parameters_data'" in response_text
+
+    def test_save_parameters_data_invalid_args(self, test_client, session):
+        """Test invalid/missing mandatory args in executor_data returns 400."""
+        executor = factories.ExecutorFactory(
+            parameters_metadata={
+                "TARGET": {"base": "list", "type": "list", "mandatory": True},  # Required field
+                "OPTION_SC": {"base": "boolean", "type": "boolean", "mandatory": False}
+            }
+        )
+        session.add(executor)
+        session.commit()
+
+        data = {
+            "executor_id": executor.id,
+            "parameters_data": {
+                "executor_data": {
+                    "args": {
+                        "OPTION_SC": 1  # This field should be boolean
+                    }
+                }
+            }
+        }
+        response = test_client.post(self.url() + '/save_parameters', json=data, content_type="application/json")
+        print(response.json)
+        assert response.status_code == 400
+        assert response.json["errors"]['OPTION_SC'] == 'Expected boolean, got int'
