@@ -57,7 +57,6 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
         assert res.status_code == 200
         assert 'rows' in res.json
         for vuln in res.json['rows']:
-            assert {'id', 'key', 'value', 'doc'} == set(vuln.keys())
             object_properties = [
                 'exploitation',
                 'references',
@@ -72,7 +71,7 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
             ]
 
             expected = set(object_properties)
-            result = set(vuln['doc'].keys())
+            result = set(vuln.keys())
             assert expected - result == set()
 
     def _create_post_data_vulnerability_template(self, references):
@@ -168,7 +167,7 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
         assert res.status_code == 200
         assert len(res.json['rows']) == filters['count']
         if filters['count'] == 1:
-            assert res.json['rows'][0]['doc']['name'] == templates[0].name
+            assert res.json['rows'][0]['name'] == templates[0].name
 
     @pytest.mark.usefixtures('ignore_nplusone')
     @pytest.mark.parametrize('filters', [
@@ -204,7 +203,7 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
         assert res.status_code == 200
         assert len(res.json['rows']) == filters['count']
         if filters['count'] == 1:
-            assert res.json['rows'][0]['doc']['name'] == templates[0].name
+            assert res.json['rows'][0]['name'] == templates[0].name
 
     @pytest.mark.skip_sql_dialect('sqlite')
     @pytest.mark.usefixtures('ignore_nplusone')
@@ -240,7 +239,7 @@ class TestListVulnerabilityTemplateView(ReadWriteAPITests, BulkUpdateTestsMixin,
         assert res.status_code == 200
         assert len(res.json['rows']) == filters['count']
         if filters['count'] == 1:
-            assert res.json['rows'][0]['doc']['name'] == templates[0].name
+            assert res.json['rows'][0]['name'] == templates[0].name
 
     def test_update_vulnerability_template(self, session, test_client):
         template = self.factory.create()
